@@ -113,14 +113,20 @@ mt_data()
     if (size < 0)
 	return TRUE;
 
-    printf("\nGetting thread file."), fflush(stdout);
+#ifdef VERBOSE
+    IF(verbose)
+	printf("\nGetting thread file."), fflush(stdout);
+#endif
     if (nntp_read((char*)&total, (long)sizeof (TOTAL)) < sizeof (TOTAL))
 	goto exit;
 
 #else /* !USE_XTHREAD */
     if ((fp = fopen(mt_name(ngname), FOPEN_RB)) == Nullfp)
 	return TRUE;
-    printf("\nReading thread file."), fflush(stdout);
+#ifdef VERBOSE
+    IF(verbose)
+	printf("\nReading thread file."), fflush(stdout);
+#endif
 
     if (fread((char*)&total, 1, sizeof (TOTAL), fp) < sizeof (TOTAL))
 	goto exit;
