@@ -96,7 +96,7 @@ char_int cmd;
     sel_item_index = 0;
     *promptbuf = '\0';
     disp_status_line = FALSE;
-    if (added_articles > 0) {
+    if (added_articles) {
 	register long i = added_articles, j;
 	register ARTICLE *ap = article_ptr(lastart - i + 1);
 	for (j = 0; j < added_articles; j++, ap++) {
@@ -123,9 +123,9 @@ display_selector:
     display_page();
 
     /* Check if there is really anything left to display. */
-    if (!sel_item_cnt && !empty_ok) { /*TODO: this may not be needed anymore */
+    if (!sel_item_cnt && !empty_ok) {
 	empty_complaint();
-	sel_ret = '+';
+	sel_ret = 'q';
 	goto sel_exit;
     }
     empty_ok = FALSE;
@@ -791,8 +791,7 @@ q does nothing.\n\n\
 	    selected_subj_cnt = selected_count = 0;
 	    return DS_DISPLAY;
 	}
-	sprintf(buf,"That command does not work in the set-unread selector.");
-	return DS_STATUS;
+	return DS_QUIT;
     case 'T':
 	if (!ThreadedGroup) {
 	    sprintf(buf,"Group is not threaded.");
